import octofuss
import string
import time
import datetime
import os, os.path
from twisted.internet.threads import deferToThread

mydeferred = deferToThread.__get__

LEASE_FILES = ['/var/lib/dhcp3/dhcpd.leases', '/tmp/dhcpd.leases']

class DhcpLeases(octofuss.Tree):
    def __init__(self, name = None):
        super(DhcpLeases, self).__init__(name, doc="DHCP Leases")
        self.leases = {}
        self.load_conf()

    def load_conf(self):
        filename = None
        for x in LEASE_FILES:
            if os.path.isfile(x):
                filename = x
                break

        data = []
        for line in open(filename).readlines():
            if line.startswith("#"):
                continue
            data.append(line.strip())
        data = " ".join(data)
        items = data.split("lease")
        items = list(map(string.strip, items))
        for i in items:
            if len(i) < 2:
                continue
            ip = i.split()[0]
            desc = i.split("{")[1].split("}")[0]
            desc = map(string.strip, desc.split(";"))[:-1]
            rdesc = {}
            rdesc['ip'] = ip
            for x in desc:
                if "hardware ethernet" in x:
                    rdesc['mac'] = x.split()[2]
                if "client-hostname" in x:
                    rdesc['hostname'] = x.split()[1][1:-1]
                if "starts" in x:
                    rdesc['start'] = " ".join(x.split()[2:])
                if "ends" in x:
                    ts = " ".join(x.split()[2:])
                    rdesc['end'] = ts
                    tt = (ts[:4], ts[5:7], ts[8:10],
                          ts[11:13], ts[14:16],ts[17:19],
                          0, 0, 0)
                    end_ts = time.mktime(list(map(int, tt)))
                    now = time.time()

                    rdesc['ended'] = (now - end_ts) > 0 and True or False

            if not rdesc.get('end', None):
                rdesc['ended'] = False
            self.leases[rdesc['ip']] = rdesc

    def llist(self, path):
        if not path:
            k = list(self.leases.keys())
            k.sort()
            return k
        else:
            if len(path) == 1:
                if path[0] in self.leases:
                    return list(self.leases[path[0]].keys())
            if len(path) == 2:
                return []

    def lhas(self, path):
        if len(path) == 1:
            if path[0] in self.leases:
                return True
        if len(path) == 2:
            if path[0] in self.leases:
                if path[1] in self.leases[path[0]]:
                    return True

    def lget(self, path):
        if len(path) == 0:
            return self.leases
        if len(path) == 1:
            if path[0] in self.leases:
                return self.leases[path[0]]
        if len(path) == 2:
            if path[0] in self.leases:
                if path[1] in self.leases[path[0]]:
                    return self.leases[path[0]][path[1]]

class MockDhcpLeases(DhcpLeases):
    def load_conf(self):
        import random
        for x in range(random.randint(5,20)):
            ip = "10.0.0.{}".format(x)

            lease = dict()
            mac = ":".join([str(hex(random.randint(0,255)))[2:] for i in range(6)])
            lease['mac'] = mac
            lease['hostname'] = 'host{}'.format(x)
            start_lease = datetime.datetime.now() - datetime.timedelta(seconds=random.randint(400,18000))
            lease['start'] = str(start_lease)
            lease['end'] = str(start_lease + datetime.timedelta(seconds=random.randint(800,40000)))
            lease['ended'] = lease['end']

            self.leases[ip] = lease

def init(**kw):
    conf = kw.get("conf", None)
    forceMock = conf and conf.getboolean("General", "forceMockup")

    leases = None
    if forceMock:
        leases = MockDhcpLeases("leases")
    else:
        filename = None
        for x in LEASE_FILES:
            if os.path.isfile(x):
                filename = x
                break

        if filename is not None:
            leases = DhcpLeases("leases")

            #@mydeferred
            #def update_all_data():
            #    lease.update_data()
            #    reactor.callLater(60, update_all_data)

    if leases:
        yield dict(
            tree = leases,
            root = '/dhcp'
            )
